@@ -3,7 +3,19 @@
 
 #include "config.h"
 
-#define ORBIT_RAND_DATA_LEN 8
+/*
+ * Our object key is 28 bytes long and looks like this:
+ *
+ * .----- adaptor prefix -----.---------- ObjectId -----------,
+ * |      4            16     |      4              4         |
+ * | adaptor idx | orb cookie | object idx | object id random |
+ */
+
+#define ORBIT_ADAPTOR_KEY_LEN     (128/8)
+#define ORBIT_ADAPTOR_PREFIX_LEN  (sizeof (CORBA_long) + ORBIT_ADAPTOR_KEY_LEN)
+
+/* We have an an auto-inc + 32 bits of poor randomness */
+#define ORBIT_OBJECT_ID_LEN 4
 
 /*
  * Exported by poa.c.
@@ -28,14 +40,11 @@ PortableServer_Current
  * Exported by poa-manager.c
  */
 void               ORBit_POAManager_register_poa   (PortableServer_POAManager  poa_mgr, 
-						    PortableServer_POA         poa,
-						    CORBA_Environment         *ev);
+						    PortableServer_POA         poa);
 
 void               ORBit_POAManager_unregister_poa (PortableServer_POAManager  poa_mgr, 
-						    PortableServer_POA         poa,
-						    CORBA_Environment         *ev);
+						    PortableServer_POA         poa);
 PortableServer_POAManager
-                   ORBit_POAManager_new            (CORBA_ORB                  orb,
-						    CORBA_Environment         *ev);
+                   ORBit_POAManager_new            (CORBA_ORB                  orb);
 
 #endif /* ORBIT_POA_H */
